'use strict';

var _get = require('babel-runtime/helpers/get')['default'];

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _slicedToArray = require('babel-runtime/helpers/sliced-to-array')['default'];

var _toArray = require('babel-runtime/helpers/to-array')['default'];

var _regeneratorRuntime = require('babel-runtime/regenerator')['default'];

var _Promise = require('babel-runtime/core-js/promise')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
    value: true
});

var _requestPromise = require('request-promise');

var _events = require('events');

var _websocket = require('websocket');

var _utils = require('./utils');

var _user = require('./user');

var _user2 = _interopRequireDefault(_user);

var _room = require('./room');

var _room2 = _interopRequireDefault(_room);

var _messages = require('./messages');

var MESSAGE_DELAY = 500;

/**
 * Showdown server connection.
 */

var Client = (function (_EventEmitter) {
    _inherits(Client, _EventEmitter);

    function Client() {
        _classCallCheck(this, Client);

        _get(Object.getPrototypeOf(Client.prototype), 'constructor', this).call(this);
        this.user = null;
        this._server = null;
        this._connection = null;
        this._messageQueue = [];
        this._lastMessage = 0;
        this._appendEvents();
        this._queueTimeout = null;
    }

    _createClass(Client, [{
        key: 'connect',

        /**
         * Connect to Showdown server.
         *
         * @param {string|Object} server - Showdown identifier of a server,
         *     address of a server, or object containing server properties.
         * @param {Object={}} options - Client options.
         * @param {boolean=false} options.debug - Show debug output.
         *
         * @returns {Promise}
         */
        value: function connect(server) {
            var _ref,
                _ref$debug,
                debug,
                args$2$0 = arguments;

            return _regeneratorRuntime.async(function connect$(context$2$0) {
                while (1) switch (context$2$0.prev = context$2$0.next) {
                    case 0:
                        _ref = args$2$0.length <= 1 || args$2$0[1] === undefined ? {} : args$2$0[1];
                        _ref$debug = _ref.debug;
                        debug = _ref$debug === undefined ? false : _ref$debug;

                        this._debug = debug;
                        context$2$0.next = 6;
                        return _regeneratorRuntime.awrap((0, _utils._parseServer)(server));

                    case 6:
                        this._server = context$2$0.sent;
                        context$2$0.next = 9;
                        return _regeneratorRuntime.awrap(this._initConnection());

                    case 9:
                    case 'end':
                        return context$2$0.stop();
                }
            }, null, this);
        }
    }, {
        key: '_initConnection',
        value: function _initConnection() {
            var client, clientPromise, _server, host, port;

            return _regeneratorRuntime.async(function _initConnection$(context$2$0) {
                var _this = this;

                while (1) switch (context$2$0.prev = context$2$0.next) {
                    case 0:
                        client = new _websocket.client();
                        clientPromise = new _Promise(function (resolve, reject) {
                            client.on('connect', resolve);
                            client.on('connectFailed', reject);
                        });
                        _server = this._server;
                        host = _server.host;
                        port = _server.port;

                        client.connect('ws://' + host + ':' + port + '/showdown/websocket');
                        context$2$0.next = 8;
                        return _regeneratorRuntime.awrap(clientPromise);

                    case 8:
                        this._connection = context$2$0.sent;

                        this.emit('connection');
                        this._connection.on('message', function (message) {
                            if (message.type === 'utf8') {
                                _this.emit('rawMessage', message.utf8Data);
                            } else {
                                throw new Error('Unrecognized message type \'' + message.type + '\'');
                            }
                        });

                    case 11:
                    case 'end':
                        return context$2$0.stop();
                }
            }, null, this);
        }
    }, {
        key: 'disconnect',

        /**
         * Disconnects from a server.
         */
        value: function disconnect() {
            this._connection.close();
        }
    }, {
        key: 'reconnect',

        /**
         * Reconnects to a server
         *
         * @returns {Promise}
         */
        value: function reconnect() {
            return _regeneratorRuntime.async(function reconnect$(context$2$0) {
                while (1) switch (context$2$0.prev = context$2$0.next) {
                    case 0:
                        this.disconnect();
                        context$2$0.next = 3;
                        return _regeneratorRuntime.awrap(this._initConnection());

                    case 3:
                    case 'end':
                        return context$2$0.stop();
                }
            }, null, this);
        }
    }, {
        key: 'login',

        /**
         * Logins as a provided user.
         *
         * @param {string} name - User name
         * @param {string} pass - User password
         *
         * @returns {Promise}
         */
        value: function login(name, pass) {
            var _$split, _$split2, challengekeyid, challenge, body, userData;

            return _regeneratorRuntime.async(function login$(context$2$0) {
                while (1) switch (context$2$0.prev = context$2$0.next) {
                    case 0:
                        context$2$0.next = 2;
                        return _regeneratorRuntime.awrap(this._loginPromise);

                    case 2:
                        _$split = context$2$0.sent[0].split('|');
                        _$split2 = _slicedToArray(_$split, 2);
                        challengekeyid = _$split2[0];
                        challenge = _$split2[1];
                        context$2$0.next = 8;
                        return _regeneratorRuntime.awrap((0, _requestPromise.post)({
                            url: 'https://play.pokemonshowdown.com/~~' + this._server._serverid + '/action.php',
                            form: { act: 'login', challengekeyid: challengekeyid, challenge: challenge, name: name, pass: pass }
                        }));

                    case 8:
                        body = context$2$0.sent;
                        userData = JSON.parse(body.substring(1));

                        this.send('trn', name + ',0,' + userData.assertion);
                        context$2$0.next = 13;
                        return _regeneratorRuntime.awrap(this._receiveEvent('raw-updateuser'));

                    case 13:
                    case 'end':
                        return context$2$0.stop();
                }
            }, null, this);
        }
    }, {
        key: '_receiveEvent',

        /**
         * @callback receiveEventCallback
         * @param {*[]} result - list of event parameters
         */

        /**
         * Return a promise that is accepted when a given event is retrieved.
         * Useful when you are waiting for a particular event to occur.
         *
         * @param {string} type - Required event type
         * @param {receiveEventCallback} condition - Condition required for
         *     a promise to resolve.
         *
         * @returns {Promise}
         */
        value: function _receiveEvent(type) {
            var _this2 = this;

            var condition = arguments.length <= 1 || arguments[1] === undefined ? function () {
                return true;
            } : arguments[1];

            return new _Promise(function (resolve) {
                function callback() {
                    for (var _len = arguments.length, result = Array(_len), _key = 0; _key < _len; _key++) {
                        result[_key] = arguments[_key];
                    }

                    if (!condition(result)) return;
                    this.removeListener(type, callback);
                    resolve(result);
                }
                _this2.on(type, callback);
            });
        }
    }, {
        key: 'send',

        /**
         * Send a command.
         *
         * @param {string} command - Command name.
         * @param {string=} - Command argument.
         * @param {string=lobby} - Room to use a command in.
         */
        value: function send(command) {
            var argument = arguments.length <= 1 || arguments[1] === undefined ? '' : arguments[1];
            var room = arguments.length <= 2 || arguments[2] === undefined ? 'lobby' : arguments[2];

            // Showdown allows this shortform for compression.
            if (room === 'lobby') {
                room = '';
            }
            this._sendRawMessage(room + '|/' + command + ' ' + argument);
        }
    }, {
        key: 'say',

        /**
         * Sends a message to a chat.
         *
         * @param {string} message - Message.
         * @param {string=lobby} room - Room to send message to.
         *
         * @returns {Promise}
         */
        value: function say(message) {
            var room = arguments.length <= 1 || arguments[1] === undefined ? 'lobby' : arguments[1];
            return _regeneratorRuntime.async(function say$(context$2$0) {
                while (1) switch (context$2$0.prev = context$2$0.next) {
                    case 0:
                        if (room === 'lobby') {
                            room = '';
                        }
                        // Messages starting with / would be interpreted as commands,
                        // to prevent that, additional slash needs to be added.
                        if (message.charAt(0) === '/') {
                            message = '/' + message;
                        }
                        // Similarly, to prevent messages starting with ! from acting
                        // like announcements, they need to be prepended with spaces.
                        // Similarly, to escape admin-only eval, space has to be used
                        // as well. Not that anyone would care about >>, but better be
                        // more secure if possible.
                        //
                        // Consistency, I tell you.
                        else if (/^!|>>>? /.test(message)) {
                            message = ' ' + message;
                        }

                        this._sendRawMessage(room + '|' + message);

                    case 3:
                    case 'end':
                        return context$2$0.stop();
                }
            }, null, this);
        }
    }, {
        key: 'joinRoom',

        /**
         * Joins a room.
         */
        value: function joinRoom(room) {
            this.send('join', room);
        }
    }, {
        key: '_createUser',
        value: function _createUser(nick) {
            return new _user2['default'](nick, this);
        }
    }, {
        key: '_createRoom',
        value: function _createRoom(name) {
            return new _room2['default'](name, this);
        }
    }, {
        key: '_sendRawMessage',
        value: function _sendRawMessage(message) {
            this._messageQueue.push(message);
            this._checkMessageQueue();
        }
    }, {
        key: '_checkMessageQueue',
        value: function _checkMessageQueue() {
            var now = +new Date();
            var MIN_TIME = this._lastMessage + MESSAGE_DELAY;
            if (this._messageQueue.length === 0) return;
            if (now >= MIN_TIME) {
                this._sendFromMessageQueue();
            } else if (!this._queueTimeout) {
                var callback = this._sendFromMessageQueue.bind(this);
                this._queueTimeout = setTimeout(callback, MIN_TIME - now);
            }
        }
    }, {
        key: '_sendFromMessageQueue',
        value: function _sendFromMessageQueue() {
            this._queueTimeout = null;
            this._forceSend(this._messageQueue.shift());
            this._checkMessageQueue();
        }
    }, {
        key: '_forceSend',
        value: function _forceSend(message) {
            this._lastMessage = +new Date();
            if (this._debug) {
                console.log('[snd] ' + message);
            }
            this._connection.send(message);
        }
    }, {
        key: '_onConnection',
        value: function _onConnection() {
            this._loginPromise = this._receiveEvent('raw-challstr');
        }
    }, {
        key: '_onRawMessage',
        value: function _onRawMessage(rawMessage) {
            if (this._debug) {
                console.log('[rcv] ' + rawMessage);
            }

            var MESSAGE_REGEX = /^(?:>([^\n]+)\n)?([\s\S]*?)$/;
            var room = undefined;
            var message = undefined;
            try {
                ;
                var _MESSAGE_REGEX$exec$slice = MESSAGE_REGEX.exec(rawMessage).slice(1);

                var _MESSAGE_REGEX$exec$slice2 = _slicedToArray(_MESSAGE_REGEX$exec$slice, 2);

                room = _MESSAGE_REGEX$exec$slice2[0];
                message = _MESSAGE_REGEX$exec$slice2[1];
            } catch (e) {
                throw new Error('Invalid message received');
            }
            if (!room) {
                room = 'lobby';
            }

            var command = undefined;
            var data = undefined;
            var COMMAND_REGEX = /^\|([^|]+)\|([\s\S]*)$/;
            try {
                ;
                var _COMMAND_REGEX$exec$slice = COMMAND_REGEX.exec(message).slice(1);

                var _COMMAND_REGEX$exec$slice2 = _slicedToArray(_COMMAND_REGEX$exec$slice, 2);

                command = _COMMAND_REGEX$exec$slice2[0];
                data = _COMMAND_REGEX$exec$slice2[1];
            } catch (e) {
                command = '';
                data = message;
            }

            this.emit('raw', command, data, room);
            this.emit('raw-' + command, data, room);
        }
    }, {
        key: '_onUpdateUser',
        value: function _onUpdateUser(message) {
            var _message$split = message.split('|');

            var _message$split2 = _slicedToArray(_message$split, 1);

            var nick = _message$split2[0];

            this.user = this._createUser(nick);
        }
    }, {
        key: '_onChatMessage',
        value: function _onChatMessage(rawChatMessage, room) {
            var _rawChatMessage$split = rawChatMessage.split('|');

            var _rawChatMessage$split2 = _toArray(_rawChatMessage$split);

            var timestamp = _rawChatMessage$split2[0];
            var user = _rawChatMessage$split2[1];

            var message = _rawChatMessage$split2.slice(2);

            room = this._createRoom(room);
            user = this._createUser(user);
            message = message.join('|');

            // Ignore messages from client itself
            if (user.equals(this.user)) return;

            var messageObject = new _messages.ChatMessage(room, user, message);
            this.emit('chat', messageObject);
            this.emit('message', messageObject);
        }
    }, {
        key: '_onPrivateMessage',
        value: function _onPrivateMessage(rawPrivateMessage) {
            var _rawPrivateMessage$split = rawPrivateMessage.split('|');

            var _rawPrivateMessage$split2 = _toArray(_rawPrivateMessage$split);

            var sender = _rawPrivateMessage$split2[0];
            var target = _rawPrivateMessage$split2[1];

            var message = _rawPrivateMessage$split2.slice(2);

            sender = this._createUser(sender);
            message = message.join('|');

            if (sender.equals(this.user)) return;

            var messageObject = new _messages.PrivateMessage(sender, message);
            this.emit('pm', messageObject);
            this.emit('message', messageObject);
        }
    }, {
        key: '_appendEvents',
        value: function _appendEvents() {
            this.on('rawMessage', this._onRawMessage.bind(this));
            this.on('raw-updateuser', this._onUpdateUser.bind(this));
            this.on('raw-c:', this._onChatMessage.bind(this));
            this.on('raw-pm', this._onPrivateMessage.bind(this));
            this.on('connection', this._onConnection.bind(this));
        }
    }]);

    return Client;
})(_events.EventEmitter);

exports['default'] = Client;
module.exports = exports['default'];
//# sourceMappingURL=index.js.map