'use strict';

var _regeneratorRuntime = require('babel-runtime/regenerator')['default'];

Object.defineProperty(exports, '__esModule', {
    value: true
});
exports.toId = toId;
exports.toRoomId = toRoomId;
exports._parseServer = _parseServer;

var _requestPromise = require('request-promise');

var _querystring = require('querystring');

var CROSS_DOMAIN = 'https://play.pokemonshowdown.com/crossdomain.php?';

/**
 * Renames a given identifier to internal identifier form used by
 * Showdown. This form is used internally in many places in Showdown
 * client, including user names, format names, all sort of Pokemon
 * related things.
 *
 * @param {string} name - Name to convert to identifier format.
 */

function toId(name) {
    return name.toLowerCase().replace(/[^a-z0-9]/g, '');
}

/**
 * Renames a given identifier to internal identifier form used by
 * Showdown for room name.
 *
 * @param {string} name - Name to convert to room identifier format.
 */

function toRoomId(name) {
    return name.toLowerCase().replace(/[^a-z0-9\-]/g, '');
}

/**
 * Ask Showdown Client to provide information about a server.
 *
 * @param {string|Object} server - Showdown identifier of a server,
 *     address of a server, or object containing server properties.
 *
 * @returns {Promise}
 */

function _parseServer(server) {
    var body, jsonConfig;
    return _regeneratorRuntime.async(function _parseServer$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                if (!(typeof server !== 'string')) {
                    context$1$0.next = 2;
                    break;
                }

                return context$1$0.abrupt('return', server);

            case 2:
                if (!/\./.test(server)) {
                    server += '.psim.us';
                }
                context$1$0.next = 5;
                return _regeneratorRuntime.awrap((0, _requestPromise.get)(CROSS_DOMAIN + (0, _querystring.stringify)({ host: server })));

            case 5:
                body = context$1$0.sent;

                if (body) {
                    context$1$0.next = 8;
                    break;
                }

                throw new Error('Server \'' + server + '\' not recognized.');

            case 8:
                jsonConfig = /var config = (.*);$/m.exec(body)[1];
                return context$1$0.abrupt('return', JSON.parse(jsonConfig));

            case 10:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this);
}
//# sourceMappingURL=utils.js.map